@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -32,7 +32,7 @@ the metadata time is not already found in the rpd_file.
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2019, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 try:
     using_injected = 'profile' in dict(__builtins__)
@@ -78,12 +78,20 @@ from raphodo.constants import (
     ExtractionProcessing, orientation_offset, thumbnail_offset, ThumbnailCacheOrigin,
     datetime_offset, datetime_offset_exiftool, thumbnail_offset_exiftool
 )
-from raphodo.camera import Camera, CameraProblemEx
+from raphodo.camera import (
+    Camera, CameraProblemEx, gphoto2_python_logging
+)
 from raphodo.cache import ThumbnailCacheSql, FdoCacheLarge
 from raphodo.utilities import (GenerateRandomFileName, create_temp_dir, CacheDirs)
 from raphodo.preferences import Preferences
 from raphodo.rescan import RescanCamera
 from raphodo.fileformats import use_exiftool_on_photo
+from raphodo.heif import have_heif_module
+
+
+def cache_dir_name(device_name: str) -> str:
+    """Generate a directory name for a temporary file cache"""
+    return 'rpd-cache-{}-'.format(device_name[:10].replace(' ', '_'))
 
 
 def split_list(alist: list, wanted_parts=2):
@@ -272,7 +280,21 @@ def preprocess_thumbnail_from_disk(rpd_file: RPDFile,
     """
 
     if rpd_file.file_type == FileType.photo:
-        if rpd_file.is_tiff():
+        if rpd_file.is_heif():
+            if have_heif_module:
+                bytes_to_read = rpd_file.size
+                if rpd_file.mdatatime:
+                    task = ExtractionTask.load_heif_directly
+                else:
+                    task = ExtractionTask.load_heif_and_exif_directly
+                processing.add(ExtractionProcessing.resize)
+                # For now, do not orient, as it seems pyheif or libheif does that automatically
+                # processing.add(ExtractionProcessing.orient)
+            else:
+                # We have no way to convert the file
+                task = ExtractionTask.bypass
+                bytes_to_read = 0
+        elif rpd_file.is_tiff():
             available = psutil.virtual_memory().available
             if rpd_file.size <= available:
                 bytes_to_read = rpd_file.size
@@ -409,6 +431,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         task = ExtractionTask.undetermined
         file_to_work_on_is_temporary = False
 
+        if rpd_file.is_mtp_device and rpd_file.file_type == FileType.video:
+            entire_file_required = True
+
         if not entire_file_required:
             # For many photos videos, extract a small part of the file and use
             # that to get the metadata
@@ -457,13 +482,16 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         self.device_name = arguments.name
         logging.info("Generating %s thumbnails for %s", len(arguments.rpd_files), arguments.name)
         if arguments.log_gphoto2:
-            gp.use_python_logging()
+            self.gphoto2_logging = gphoto2_python_logging()
 
         self.frontend = self.context.socket(zmq.PUSH)
         self.frontend.connect("tcp://localhost:{}".format(arguments.frontend_port))
 
         self.prefs = Preferences()
 
+        # Whether we must use ExifTool to read photo metadata
+        force_exiftool = self.prefs.force_exiftool
+
         # If the entire photo or video is required to extract the thumbnail, which is determined
         # when extracting sample metadata from a photo or video during the device scan
         entire_photo_required = arguments.entire_photo_required
@@ -475,7 +503,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         thumbnail_caches = GetThumbnailFromCache(use_thumbnail_cache=use_thumbnail_cache)
 
         photo_cache_dir = video_cache_dir = None
-        cache_file_from_camera = False
+        cache_file_from_camera = force_exiftool
 
         rpd_files = arguments.rpd_files
 
@@ -513,13 +541,21 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                     "Prematurely exiting thumbnail generation due to lack of access to camera %s",
                     arguments.camera
                 )
+                self.content = pickle.dumps(
+                    GenerateThumbnailsResults(
+                        scan_id=arguments.scan_id,
+                        camera_removed=True,
+                    ), pickle.HIGHEST_PROTOCOL
+                )
+                self.send_message_to_sink()
                 self.disconnect_logging()
                 self.send_finished_command()
                 sys.exit(0)
 
             if not cache_file_from_camera:
                 for rpd_file in rpd_files:
-                    if use_exiftool_on_photo(rpd_file.extension):
+                    if use_exiftool_on_photo(rpd_file.extension,
+                                             preview_extraction_irrelevant=False):
                         cache_file_from_camera = True
                         break
 
@@ -532,10 +568,10 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 # the download process
                 self.photo_cache_dir = create_temp_dir(
                     folder=arguments.cache_dirs.photo_cache_dir,
-                    prefix='rpd-cache-{}-'.format(self.device_name[:10]))
+                    prefix=cache_dir_name(self.device_name))
                 self.video_cache_dir = create_temp_dir(
                     folder=arguments.cache_dirs.video_cache_dir,
-                    prefix='rpd-cache-{}-'.format(self.device_name[:10]))
+                    prefix=cache_dir_name(self.device_name))
                 cache_dirs = CacheDirs(self.photo_cache_dir, self.video_cache_dir)
                 self.content = pickle.dumps(
                     GenerateThumbnailsResults(
@@ -581,8 +617,6 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
             if task != ExtractionTask.undetermined:
                 if origin == ThumbnailCacheOrigin.thumbnail_cache:
                     from_thumb_cache += 1
-                    # logging.debug("Thumbnail for %s found in RPD thumbnail cache",
-                    #               rpd_file.full_file_name)
                 else:
                     assert origin == ThumbnailCacheOrigin.fdo_cache
                     logging.debug(
@@ -607,7 +641,20 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 # Thumbnail was not found in any cache: extract it
                 if self.camera:  # type: Camera
                     if rpd_file.file_type == FileType.photo:
-                        if self.camera.can_fetch_thumbnails:
+                        if rpd_file.is_heif():
+                            # Load HEIF / HEIC using entire file.
+                            # We are assuming that there is no tool to extract a
+                            # preview image from an HEIF / HEIC, or the file simply
+                            # does not have one to extract.
+                            if self.cache_full_size_file_from_camera(rpd_file):
+                                task = ExtractionTask.load_heif_and_exif_directly
+                                processing.add(ExtractionProcessing.resize)
+                                full_file_name_to_work_on = rpd_file.cache_full_file_name
+                                # For now, do not orient, as it seems pyheif or libheif does
+                                # that automatically.
+                                # processing.add(ExtractionProcessing.orient)
+
+                        elif self.camera.can_fetch_thumbnails:
                             task = ExtractionTask.load_from_bytes
                             if rpd_file.is_jpeg_type():
                                 # gPhoto2 knows how to get jpeg thumbnails
@@ -619,7 +666,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                                     # TODO handle error?
                                     thumbnail_bytes = None
                             else:
-                                if use_exiftool_on_photo(rpd_file.extension):
+
+                                if force_exiftool or use_exiftool_on_photo(rpd_file.extension,
+                                                           preview_extraction_irrelevant=False):
                                     task, full_file_name_to_work_on, \
                                         file_to_work_on_is_temporary =\
                                         self.extract_photo_video_from_camera(
@@ -666,6 +715,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                             # from it (which probably doesn't exist!). This is fast.
                             # If have rawkit, download and render an image from the
                             # RAW
+
                             if not rpd_file.is_jpeg() and not have_rawkit:
                                 bytes_to_read = thumbnail_offset.get(rpd_file.extension)
                                 if bytes_to_read:
@@ -765,7 +815,8 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                         use_thumbnail_cache=use_thumbnail_cache,
                         file_to_work_on_is_temporary=file_to_work_on_is_temporary,
                         write_fdo_thumbnail=False,
-                        send_thumb_to_main=True
+                        send_thumb_to_main=True,
+                        force_exiftool=force_exiftool
                     ),
                     pickle.HIGHEST_PROTOCOL)
                 self.frontend.send_multipart([b'data', self.content])
